"""
Simple interface to SyncRNG. This file defines a SyncRNG object which can be 
used to seed and pull numbers from the RNG.

"""

from copy import deepcopy
from warnings import warn

from _syncrng import seed as _seed
from _syncrng import rand as _rand


class SyncRNG(object):
    def __init__(self, seed=0):
        self.BPF = 32
        self.seed = seed
        self.state = _seed(self.seed)

    def randi(self):
        tmp = _rand(self.state)
        self.state = tmp[:-1]
        return int(tmp[-1])

    def rand(self):
        return self.randi() * 2.3283064365387e-10

    def randbelow(self, n):
        maxsize = 1 << self.BPF
        if n >= maxsize:
            warn(
                "Underlying random generator does not supply \n"
                "enough bits to choose from a population range this "
                "large.\n"
            )
            return int(self.rand() * n)
        rem = maxsize % n
        limit = (maxsize - rem) / maxsize
        r = self.rand()
        while r >= limit:
            r = self.rand()
        return int(r * maxsize) % n

    def shuffle(self, x):
        y = deepcopy(x)
        for i in reversed(range(1, len(y))):
            j = self.randbelow(i + 1)
            y[i], y[j] = y[j], y[i]
        return y



# -*- coding: utf-8 -*-

import os
import unittest

from SyncRNG import SyncRNG


class SyncRNGTestCase(unittest.TestCase):
    def test_randi(self):
        s = SyncRNG(seed=123456)
        self.assertEqual(s.randi(), 959852049)
        self.assertEqual(s.randi(), 2314333085)
        self.assertEqual(s.randi(), 2255782734)
        self.assertEqual(s.randi(), 2921461239)
        self.assertEqual(s.randi(), 1024197102)

    def test_rand(self):
        s = SyncRNG(seed=123456)
        self.assertAlmostEqual(s.rand(), 959852049 / pow(2, 32))
        self.assertAlmostEqual(s.rand(), 2314333085 / pow(2, 32))
        self.assertAlmostEqual(s.rand(), 2255782734 / pow(2, 32))
        self.assertAlmostEqual(s.rand(), 2921461239 / pow(2, 32))
        self.assertAlmostEqual(s.rand(), 1024197102 / pow(2, 32))

    def test_randbelow(self):
        s = SyncRNG(seed=123456)
        self.assertEqual(s.randbelow(5), 4)
        self.assertEqual(s.randbelow(5), 0)
        self.assertEqual(s.randbelow(5), 4)
        self.assertEqual(s.randbelow(5), 4)
        self.assertEqual(s.randbelow(5), 2)

    def test_shuffle(self):
        s = SyncRNG(seed=123456)
        x = [1, 2, 3, 4, 5]
        y = s.shuffle(x)
        self.assertEqual(y, [3, 4, 1, 2, 5])

    def test_first_1000(self):
        s = SyncRNG(seed=0)

        here = os.path.dirname(__file__)
        test_file = os.path.join(here, "first_1000_seed_0.txt")

        with open(test_file, "r") as fid:
            for line in fid:
                exp = int(line.strip())
                self.assertTrue(exp == s.randi())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Setup file for SyncRNG

Author: Gertjan van den Burg
Date: Oct. 12, 2016
"""

import io
import os

from setuptools import find_packages, setup
from distutils.extension import Extension

# Package meta-data.
AUTHOR = "Gertjan van den Burg"
DESCRIPTION = "Generate the same random numbers in R and Python"
EMAIL = "gertjanvandenburg@gmail.com"
LICENSE = "GPLv2"
LICENSE_TROVE = (
    "License :: OSI Approved :: GNU General Public License v2 (GPLv2)"
)
NAME = "SyncRNG"
REQUIRES_PYTHON = ">=3.6.0"
URL = "https://github.com/GjjvdBurg/SyncRNG"
VERSION = None

# What packages are required for this module to be executed?
REQUIRED = []

docs_require = []
test_require = []
dev_require = ["green"]

# What packages are optional?
EXTRAS = {
    "docs": docs_require,
    "test": test_require,
    "dev": docs_require + test_require + dev_require,
}

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!

# We don't use try-except here because we want builds to fail if they can't 
# find the README.
with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, "__version__.py")) as f:
        exec(f.read(), about)
else:
    about["__version__"] = VERSION

# Where the magic happens:
setup(
    name=NAME,
    version=about["__version__"],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(
        exclude=["tests", "*.tests", "*.tests.*", "tests.*"]
    ),
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license=LICENSE,
    ext_modules=[
        Extension(
            "_syncrng",
            define_macros=[("TARGETPYTHON", "1")],
            sources=["src/_syncrng.c"],
            extra_compile_args=["-g"],
        )
    ],
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        LICENSE_TROVE,
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Mathematics",
    ],
)
